import processing.video.*;

Capture cam;
int current = 13;

int index = 0;
ArrayList<PVector> selected = new ArrayList<PVector>();

void setup() {
  size(640, 480, P2D);
  rectMode(CORNERS);
  
  String[] cameras = Capture.list();
  
  if (cameras.length == 0) {
    println("There are no cameras available for capture.");
    exit();
  } else {
    println("Available cameras:");
    for (int i = 0; i < cameras.length; i++)
      println(i, cameras[i]);
    
    // The camera can be initialized directly using an 
    // element from the array returned by list():
    cam = new Capture(this, cameras[current]);
    cam.start();
  }      
}

void exit() {
  PrintWriter output = createWriter("regions.txt");
  for (int n = 0; n < selected.size(); n++)
    if (n % 2 == 1)
      output.println(selected.get(n).x + ", " + selected.get(n).y + ", " + selected.get(n - 1).x + ", " + selected.get(n - 1).y);
  output.flush();
  output.close();
  super.exit();
}

void draw() {
  if (selected.size() <= index)
      selected.add(new PVector(0,0));
  
  if (cam.available() == true)
    cam.read();
  image(cam, 0, 0);
  
  noFill();
  strokeWeight(2);
  stroke(index % 2 == 0 ? #FFAA44 : #44AAFF);
  ellipse(selected.get(index).x, selected.get(index).y, 10, 10);
  
  for (int n = 0; n < selected.size(); n++)
    if (n % 2 == 1) {
      noStroke();
      fill((n / 2) % 2 == 0 ? 0xAA000000 : 0xAAFFFFFF);
      rect(selected.get(n).x, selected.get(n).y, selected.get(n - 1).x, selected.get(n - 1).y);
    }
}

void mouseClicked() {
  selected.set(index, new PVector(mouseX, mouseY));
}

void keyPressed() {
  if (key == ENTER || key == RETURN)
    index++;
  if (key == DELETE && index > 0)
    index--;
}